#ifndef SHAPE_H_
#define SHAPE_H_
#include "type.h"

class Edge;
class Vert;
class Medium;
class Conductor;
struct LinEle;
class SourcePoint;
class Element;

typedef enum {OBJECT,SHELL }DataStructLayer;
typedef enum {SEPERATE,IN,OUT,ADJOIN,MERGE,MERGEIN,MERGEOUT,INTERSECT} ShellRelation;
typedef enum {SHAREARC,DISCONNECT,CONNECT,SAMESTART,SAMEEND,REVERSECONNECT,SHARELINE,CROSS} EdgeRelation;
typedef enum {INTURN,       // --->--->
	      REVERSE,      // <---<---
	      FACE,         // ---><---
	      OPPOSITE      // <------>
	      }ShareLineDircRela;
typedef enum {CLOCKWISE,ANTICLOCKWISE} CrossDircRela;
typedef enum {GREATER,EQUAL,LESS} FloatValueRelation;
typedef enum {POSITIVE,NEGATIVE} LineDirection;
typedef enum {NONE,MEDIUM,CONDUCTOR } ObjElecType;
typedef enum {BOX,POLY,POINTS,EDGES}ShapeType;

typedef struct {
		 double x,y;
		 }Point;
typedef struct {
		 double an;
		 Flag deflt;
		 }Ang;
typedef struct {
		 double x1,y1,x2,y2,x3,y3,x4,y4;
            //leftdown:x1,y1
            //rightdown:x2,y2
            //rightup:x3,y3
            //leftup:x4,y4
		 double width,height;
		 int elesonwidth,elesonheight;
		 }Box;
typedef struct {
         Point *vertex;
		 int *eles;
		 int vertnum;
		 }Poly;
typedef struct {
         Point *vertex;
		 int *eles;
		 int vertnum;
		 double d1,d2;
		 }Points;
typedef union {
		Box *bbox;
		Poly *ppoly;
		Points *ppoints;
		}ShapeUnion;
struct ShapeEdge_{
		Edge *ed;
		Vert **po;   // to decide valid part of a shape edge
		int pointnum;
		struct ShapeEdge_ *next;
		};
typedef struct ShapeEdge_ ShapeEdge;
struct Object_{
		 int objno;
		 char name[15];
		 ObjElecType electype;
		 int shellnum;
		 struct Shell_ **shell;
		 double voltval;
		 double dielval;
		 double rhoval;
		 int mediumno;    // to decide ed->medium and ed->reversemed
				  //before condandmed()
		 double minx,miny,maxx,maxy;  //box shell of object
		};
typedef struct Object_ Object;
struct Shell_{
		 double minx,miny,maxx,maxy;
		 ShapeType shapetype;
		 ShapeUnion shape;
		 ShapeEdge *edge;            //edges of this shape
		 int shellno;
		 int edgenum;                //How many edges
		 Object *obj;                 //which object it belongs to
		 };
typedef struct Shell_ Shell;


//part 1 in shape.c
void initobj(Object *);
void copyshell(Shell *,Shell *);
void copypoly(Poly *,Poly *);
void copypoints(Points *,Points *);
void printobj(Object *);
void printshell(Shell *shell);
void printtwoshapesrelation(ShellRelation sr);
void printpoint(Vert *);
void printrelation(ShellRelation **,int);
void setboxshell(Shell *,Box *);
void setpolyshell(Shell *,Poly *);
void printbox(Box *);
void printpoly(Poly *);

FloatValueRelation fltvalrela(double,double);
ShellRelation setshellrelation(Shell *,Shell *);
ShellRelation setobjectrelation(Object *,Object *);

//part 2 in shapecalc.c
void printshapeedge(ShapeEdge *);         //shapecalc.c
void printanedge(Edge *);                  //shapecalc.c
void createedgeforshape(Object *,Shell *);
void createedgeforbox(Object *o,Shell *s,Box *b);
void createedgeforpoly(Object *o,Shell *s,Poly *p);
void createvertforedge(Edge *,double,double,double,double);  
Vert *searchvert(double,double);
Vert *createnewvert();
ShapeEdge *createshapeedge();
Edge* createnewedge();
Edge* createnewforceedge();
Edge* createnewmededge(Edge *);

void shapesinsameobj(Shell *,Shell *,ShellRelation sr);
void shapesindiffmed(Shell *,Shell *,ShellRelation sr);
void shapesindiffcond(Shell *,Shell *,ShellRelation sr);
Bool condandmed(Medium *,Shell *,Shell *,ShellRelation sr);
Bool cmINTER(Medium *m,Shell *,Shell *);
Bool condandmedshareline(Medium *,ShapeEdge *,Edge *,Edge *);
Edge *condandmedcross(Medium *,Shell *,ShapeEdge *,ShapeEdge *,Edge *,Edge *,Vert **,Bool *);
Flag sharelineinsameobj(Shell *,Shell *,ShapeEdge *,ShapeEdge *,Edge *,Edge *);
void crossinsameobj(ShapeEdge *,ShapeEdge *,Edge *,Edge *);
void sharelineindiffmed(Shell *,Shell *,ShapeEdge *,ShapeEdge *,Edge *,Edge *);

EdgeRelation edgerelation(Edge *e1,Edge *e2);
Vert *crosspoint(Edge *e1,Edge *e2);
int sharelinetype(Edge *e1,Edge *e2);
ShareLineDircRela sharelinedircrela(Edge *e1,Edge *e2);
CrossDircRela crossdircrela(Edge *,Edge *);
LineDirection linedirection(Edge *ed);
Flag pointLess(Vert *s,Vert *e);
Bool isshareline(Edge *e1,Edge *e2);
void setequationofline(Edge *ed);
void setlinescope(Edge *ed);
void setedgescope(Edge *ed);
ShapeEdge * remove(ShapeEdge *s,ShapeEdge *c);
int removeresidualedge(Shell *);
Edge * addedgetomed(Medium *,Edge *);
Edge *deledgefrommed(Medium *,Edge *,Vert *,Vert *);
int searchedge(Medium *,int ,Vert *,Vert *);
void addedgetocond(Medium *,Conductor *,Edge *);
Bool edgeisinshell(Edge *,Shell *);
Bool pointinsideshell(double,double,Shell *);
Bool pointonshell(double,double,Shell *);
Bool pointinsideedge(double,double,Edge *);
Bool pointonedge(double,double,Edge *);
ShapeEdge *addedge(ShapeEdge *,ShapeEdge *,ShapeEdge *,Flag);	
	// 0 means after, 1 before
void filledgefield(Edge *,Edge *);
void addpoint(ShapeEdge *,Vert *);
Vert *getvert(ShapeEdge *,Vert *,Vert *);
void persue(double *,double *,double *,double *,int,int);


//These function are part 3 in "bem.c"
void createelement(Medium *,Edge *);
void countvarsimple(Medium *,Edge *ed);
void countvarborderq(Medium *,Edge *ed);
void countvarborderu(Medium *,Edge *ed);
void calextra(Vert *);
int getUvar(Medium *,Vert *,int,Element,EdgeType);
int getQvar(Vert *,int);
int integral(Medium *,double *,double **,SourcePoint *,int);
void buildsp(Medium *,SourcePoint *);
int newsp(SourcePoint *,int,double,double,int,int);
double *calgpoint(Edge *,LinEle *,SourcePoint *,int *,int *);
void setgvalue();
void I_integral(double *,double *,Edge *);
Edge * searchborder(Edge *);
void sortedge(Medium *);
Bool samepoint(Vert *s,Vert *e);
void solveequation(double **,double **,double **);
void gausssolve(double **,double **,double **);
void lapacksolve(double **,double **,double **);
void countedgenuminmed(Medium *,Edge *);
Bool fullmatrix(double **,double **,double **);
void result(double **,double **);
void printmatrix11(double **,int,int);

//output.c
void out(double **);

//free.c
void freespace();

#endif
