#include <unistd.h>
#include <sys/times.h>
#include <stdlib.h>
#include <assert.h>
#include "lapacke.h"
#include "shape.h"
#include "bem.h"
#include "globalvar.h"

void bem()
{
	int i, j, k, i1;
	Edge *ed;
	Vert *v;
	double **b = NULL, **r = NULL, **x = NULL;

	for ( i = 0; i < vertnum; i++ ) {
		v = vert[i / 10] + i % 10;
		v->edgenum = 0;
		v->forcenum = 0;
		v->bordernum = 0;
		v->edge = ( Edge ** ) malloc ( sizeof ( Edge * ) );
		v->varnum = -1;
		v->Uvarno = 0;
		v->feedback = NULL;
	}

	//Remove residual edges
	for ( i = 0; i < mednum; i++ ) {
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			if ( ed->electype != BORDER )
				ed->reversemed = -1;
			if ( ed->electype == BORDER && ed->reversemed == i ) {
				med[i].edge[j] = searchborder ( ed );
			}
		}
	}
	for ( i = 0; i < mednum; i++ ) {
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			countedgenuminmed ( med + i, ed );	//Just count edgenum, and should
												//count in createele again, otherwise it
												//will influence v->edge, v->sno, and so
		}
		i1 = med[i].edgenum;
		do {
			med[i].edgenum = i1;
			i1 = 0;
			for ( j = 0; j < med[i].edgenum; j++ ) {
				ed = med[i].edge[j];
				if ( ed->s->edgenum < 2 || ed->e->edgenum < 2 ) {
					ed->s->edgenum--;
					ed->e->edgenum--;
					printanedge ( ed );
					printf ( "is removed as redundant.\n" );
				}
				else
					med[i].edge[i1++] = ed;
			}
		}
		while ( i1 < med[i].edgenum );
		if ( i1 == 0 ) ;			//If no edge in this medium now
		for ( j = 0; j < vertnum; j++ )
			vert[j / 10][j % 10].edgenum = 0;
	}
	int totalelenum = 0;
	for ( i = 0; i < mednum; i++ ) {
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			createelement ( med + i, ed );
			//zcs.1211.if ( ed->edgetype != CURVE ) {
				totalelenum += ed->elenum;
			//}
			if ( ed->elenum > 100 || ed->elenum == 0 )
				printanedge ( ed );
		}
	}
	if ( totalelenum > 10000 )
		printf ( "Warning: total number of elements (%d) exceed 10000!\n",
				 totalelenum );
	for ( i = 0; i < vertnum; i++ ) {
		v = vert[i / 10] + i % 10;
		v->Qvarno = ( int * ) malloc ( v->edgenum * sizeof ( int ) );
		for ( j = 0; j < v->edgenum; j++ )
			if ( v->edge[j]->electype == NATURAL )
				v->Qvarno[j] = -v->edgenum - 1;
			else
				v->Qvarno[j] = -v->edgenum;
	}
	for ( i = 0; i < mednum; i++ )
		sortedge ( med + i );
	for ( i = 0; i < mednum; i++ )
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
		}
	gbvarno = 0;
	spno = 0;
	setgvalue();
	b = ( double ** ) malloc ( condobjnum * sizeof ( double * ) );
	x = ( double ** ) malloc ( condobjnum * sizeof ( double * ) );
	r = ( double ** ) malloc ( condobjnum * sizeof ( double * ) );
	//Because r is released later than b&x
	for ( i = 0; i < condobjnum; i++ ) {
		r[i] = ( double * ) malloc ( condobjnum * sizeof ( double ) );
		memset ( r[i], 0, ( condobjnum ) * sizeof ( double ) );
	}

	fullmatrix ( b, x, r );

	if ( condobjnum != 0 ) {
		free ( ( char * ) b );
		free ( ( char * ) x );
	}
	smgno=0;
	exit(0);
}

//Sort edge as:
//Force and Natural, Border between med[0], Border between med[1], ... , Border between med[n-1]
void sortedge ( Medium *m )
{
	int i, j, k;
	Edge *ed;
	int m1, m2;

	for ( i = 0; i < m->edgenum - 1; i++ ) {
		k = i;
		for ( j = i + 1; j < m->edgenum; j++ ) {
			if ( m->edge[k]->electype == BORDER
					&& m->edge[k]->medium == m->mediumno )
				m1 = m->edge[k]->reversemed;
			else
				m1 = m->edge[k]->medium;
			if ( m->edge[j]->electype == BORDER
					&& m->edge[j]->medium == m->mediumno )
				m2 = m->edge[j]->reversemed;
			else
				m2 = m->edge[j]->medium;
			if ( m1 > m2 )
				k = j;
		}
		if ( k != i ) {
			ed = m->edge[i];
			m->edge[i] = m->edge[k];
			m->edge[k] = ed;
		}
	}
}

void solveequation ( double **a, double **b, double **x )
{
	if (gaussflag==FALSE)
		lapacksolve ( a, b, x );
	else
		gausssolve (a, b, x);
}

void gausssolve (double **a, double **b, double **x)
{
  printf("\nUse Gauss Solve.");
  int i, j, ii, k;
  double t1, t2;
  double *temp;
  double *a1, *a2, *b1, *b2, *x1;

  a1 = a[0];
  temp = (double *) malloc (gbvarno * sizeof (double));
  for (i = 0; i < gbvarno; i++)
    {
      t1 = fabs (a1[i]);
      k = i;
      a2 = a1;
	  
      //search col master element
      for (j = i + 1; j < gbvarno; j++)
			{
	  		a2 = a2 + gbvarno;
	  		t2 = fabs (a2[i]);
	  		if (Less (t1, t2))  //t2-t1>10e-5
	    	{
	      	k = j;
	      	t1 = t2;
	    	}
			}
      //found the greatest one on ith column below right a[i][i]
      //then change ith row and kth row

      if (k != i)
	{
	  //row change of a
	  a2 = a[k];
	  memcpy (temp, a1, sizeof (double) * gbvarno);
	  memcpy (a1, a2, sizeof (double) * gbvarno);
	  memcpy (a2, temp, sizeof (double) * gbvarno);

	  //row change of b
	  for (ii = 0; ii < condobjnum; ii++)
	    {
	      t1 = b[ii][i];
	      b[ii][i] = b[ii][k];
	      b[ii][k] = t1;
	    }
	}
      //ith row of a and b both /t1

      t1 = a1[i];
      for (k = i; k < gbvarno; k++)
	a1[k] = a1[k] / t1;
      for (ii = 0; ii < condobjnum; ii++)
	b[ii][i] = b[ii][i] / t1;
      //Through row-Xiao Qu,
      //change places below a[i][i] to 0, i.e matrix to up-trangle
      a2 = a1 + gbvarno;
      for (j = i + 1; j < gbvarno; j++)	//from (i+1)th row to the last one
	{
//      if((a2[i])>=EPS || a2[i]<-EPS)
	  if (fabs (a2[i]) > EPS)
	    {
	      t1 = a2[i];
	      for (k = i; k < gbvarno; k++)
//        a2[k]=a2[k]/t1-a1[k];   //number below a[i][i] becomes 0.0
		a2[k] -= a1[k] * t1;	// * is faster than /
	      for (ii = 0; ii < condobjnum; ii++)
//        b[ii][j]=b[ii][j]/t1-b[ii][i];
		b[ii][j] -= b[ii][i] * t1;	// * is faster than /
	    }
	  a2 += gbvarno;
	}
      a1 += gbvarno;
//    printmatrix(a,gbvarno,gbvarno);
    }

  for (i = 0; i < condobjnum; i++)	//col
    {
      a1 = a[gbvarno - 1];
      for (ii = gbvarno - 1; ii >= 0; ii--)	//row
	{
	  x[i][ii] = b[i][ii];
	  for (j = gbvarno - 1; j > ii; j--)
	    x[i][ii] -= a1[j] * x[i][j];
	  a1 -= gbvarno;
	}
    }

  free ((char *) temp);
}

void lapacksolve ( double **a, double **b, double **x )
{
  	printf("\nUse Lapack Solve.");
	int info;
	int i, j, *ipiv;
	double *a1, *b1, *atemp, *btemp;

	atemp = ( double * ) malloc ( ( gbvarno * gbvarno ) * sizeof ( double ) );
	btemp = ( double * ) malloc ( ( gbvarno * condobjnum ) * sizeof ( double ) );
	ipiv = ( int * ) malloc ( gbvarno * sizeof ( int ) );

	for ( i = 0; i < gbvarno; i++ ) {
		a1 = a[i];
		for ( j = 0; j < gbvarno; j++ ) {
			atemp[i * gbvarno + j] = a1[j];
		}
	}
	for ( i = 0; i < gbvarno; i++ ) {
		for ( j = 0; j < condobjnum; j++ ) {
			btemp[i * condobjnum + j] = b[j][i];
		}
	}
	info =
		LAPACKE_dgesv ( LAPACK_ROW_MAJOR, gbvarno, condobjnum, atemp, gbvarno,
						ipiv, btemp, condobjnum );
	if ( info > 0 ) {
		printf ( "The diagonal element of the triangular factor of A,\n" );
		printf ( "U(%i,%i) is zero,so that A is singular;\n", info, info );
		printf ( "the solution could not be computed.\n" );
		exit ( 1 );
	}
	for ( i = 0; i < gbvarno; i++ ) {
		for ( j = 0; j < condobjnum; j++ ) {
			x[j][i] = btemp[i * condobjnum + j];
		}
	}
	free ( ( double * ) atemp );
	free ( ( double * ) btemp );
	free ( ( int * ) ipiv );
}

int integral ( Medium *m, double *a1, double **b, SourcePoint *sp, int linenum )
{
	int spstart;
	int gpnum, kk;
	int ie, i, j, k;
	SourcePoint *p2;
	double cs, g1, g2, h1, h2, l, dist, dxx, dyy, rr, flag, qq, uu;
	Edge *ed;
	Flag type;
	LinEle *le;
	double *gpoint;
	double *g, *fa1, *fa2;
	spstart = spno;
	buildsp ( m, sp );
	m->spstart = linenum;
	for ( i = spstart; i < spno; i++ ) { //Every Source Point i
		p2 = sp + i;
		cs = 0;
		for ( j = 0; j < m->edgenum; j++ ) { //Every edge j
			ed = m->edge[j];
			if ( ed->electype != BORDER )
				flag = 1.0;
			else if ( ed->medium == m->mediumno )
				flag = 1 / m->diel;
			else
				flag = -1 / m->diel;
			
			//case edgetype
					l = ed->l;
					for ( ie = 0; ie < ed->elenum; ie++ ) { //Every element ie
						le = ed->ele + ie;
						h1 = h2 = g1 = g2 = 0;
						dxx = le->xs - p2->x;
						dyy = le->ys - p2->y;

						//dist is the vertical distance between p2 and le(i.e. ed)
						dist = ( ( ed->dy * dxx - ed->dx * dyy ) );

						//if dist==0, dr/dn is 0, u * q* is zero
						//dist=fabs(((ed->dx)*(p2->y)-ed->dy*(p2->x)-le->ys*ed->dx+le->xs*ed->dy)/l);
						//Here is the real dist.
						//modifiedmark
						//dist==0
						//dy/dx=dyy/dxx
						//p2 and le on a same line
						if ( fabs ( dist ) < EPS )
							if ( ed->electype == NATURAL )
								type = 1;
							else
								type = 2;
						else {
							if ( flag < 0 )
								dist = -dist;
							if ( ed->electype == NATURAL )
								type = 3;
							else
								type = 4;
						}
						switch ( type ) {
							case 1:
								break;
							case 2:
								//q * -ln(r)/2PIE
								if ( Equal ( p2->x, le->xe ) && Equal ( p2->y, le->ye ) ) {
									//   --.
									g2 = 3 - 2 * log ( l );
									g1 = g2 - 2;
								}
								else if ( Equal ( p2->x, le->xs ) && Equal ( p2->y, le->ys ) ) {
									//  .--
									g1 = 3 - 2 * log ( l );
									g2 = g1 - 2;
								}
								//p2 is middle point of le
								//   -.-
								else if ( Equal ( p2->x, ( le->xs + le->xe ) / 2 )
										  && Equal ( p2->y, ( le->ys + le->ye ) / 2 ) ) {
									g1 = 2 - 2 * log ( l / 2 );
									g2 = g1;
								}
								//regular point
								//  .--    or    --.
								else {
									gpoint = calgpoint ( ed, le, p2, &gpnum, &kk );
									g = gic + kk;
									fa1 = fa11 + kk;
									fa2 = fa12 + kk;
									for ( k = 0; k < gpnum; k++ ) {
										dxx = gpoint[2 * k] - p2->x;
										dyy = gpoint[2 * k + 1] - p2->y;
										rr = dyy * dyy + dxx * dxx;
										uu = g[k] * log ( rr );
										//u*
										g1 -= uu * fa2[k];
										//g1g2
										g2 -= uu * fa1[k];
									}
								}
								if ( flag != 1.0 ) {
									g1 = flag * g1;
									g2 = flag * g2;
								}
								a1[le->qs] -= g1 * 0.25 * l;
								a1[le->qe] -= g2 * 0.25 * l;
								break;
							case 3:
								//u * -1/(2*PIE*r)*(dr/dn)
								gpoint = calgpoint ( ed, le, p2, &gpnum, &kk );
								g = gic + kk;
								fa1 = fa11 + kk;
								fa2 = fa12 + kk;
								for ( k = 0; k < gpnum; k++ ) {
									dxx = gpoint[2 * k] - p2->x;
									dyy = gpoint[2 * k + 1] - p2->y;
									rr = dyy * dyy + dxx * dxx;
									qq = g[k] / rr;
									//q*
									h1 += qq * fa2[k];
									//h1h2
									h2 += qq * fa1[k];
								}
								h1 = h1 * ( -0.5 ) * dist;
								h2 = h2 * ( -0.5 ) * dist;
								cs = cs - h1 - h2;
								if ( le->us >= 0 ) {
									a1[le->us] += h1;
								}
								//b[port][line]
								else {
									b[- ( le->us + 1 )][linenum] -= h1;
								}
								if ( le->ue >= 0 ) {
									a1[le->ue] += h2;
								}
								else {
									b[- ( le->ue + 1 )][linenum] -= h2;
								}
								break;
							case 4:
								//u * q* + q * u*
								gpoint = calgpoint ( ed, le, p2, &gpnum, &kk );
								g = gic + kk;
								fa1 = fa11 + kk;
								fa2 = fa12 + kk;
								for ( k = 0; k < gpnum; k++ ) {
									dxx = gpoint[k * 2] - p2->x;
									dyy = gpoint[k * 2 + 1] - p2->y;
									rr = dyy * dyy + dxx * dxx;
									qq = g[k] / rr;
									//q*
									h1 += qq * fa2[k];
									//h1h2
									h2 += qq * fa1[k];
									uu = g[k] * log ( rr );
									//u*
									g1 -= uu * fa2[k];
									//g1g2
									g2 -= uu * fa1[k];
								}
								h1 = h1 * ( -0.5 ) * dist;
								h2 = h2 * ( -0.5 ) * dist;
								cs = cs - h1 - h2;
								if ( le->us >= 0 ) {
									a1[le->us] += h1;
								}
								//b[port][line]
								else {
									b[- ( le->us + 1 )][linenum] -= h1;
								}
								if ( le->ue >= 0 ) {
									a1[le->ue] += h2;
								}
								else {
									b[- ( le->ue + 1 )][linenum] -= h2;
								}
								if ( flag != 1.0 ) {
									g1 = g1 * flag;
									g2 = g2 * flag;
								}
								a1[le->qs] -= g1 * 0.25 * l;
								a1[le->qe] -= g2 * 0.25 * l;
								break;
						}
					}
		}
		//cs*us
		if ( p2->u2 < -condobjnum )
			if ( p2->u1 < 0 ) {
				b[- ( p2->u1 + 1 )][linenum] -= cs;   // p2 on force edge
			}
			else {
				a1[p2->u1] += cs;
			}
		else {
			//additional sp
			if ( p2->u1 < 0 ) {
				b[- ( p2->u1 + 1 )][linenum] -= cs / 2;
			}
			else {
				a1[p2->u1] += cs / 2;
			}
			if ( p2->u2 < 0 ) {
				b[- ( p2->u2 + 1 )][linenum] -= cs / 2;
			}
			else {
				a1[p2->u2] += cs / 2;
			}
		}
		a1 += gbvarno;
		linenum++;
	}
	m->spend = linenum - 1;
	return linenum;
}

void I_integral ( double *x, double *r, Edge *ed )
{
	int i, j;
	LinEle *le;
	double q1, q2, q3;
	double vv, ww;
	double *gi, *fa1, *fa2, *fa3;

	if ( ed->electype != FORCE )
		return;
	gi = gic + 21;
	vv = 0;
			fa1 = fa11 + 21;
			fa2 = fa12 + 21;
			for ( i = 0; i < ed->elenum; i++ ) {
				le = ed->ele + i;
				ww = 0;
				if ( le->qs >= 0 )
					q1 = x[le->qs];
				else
					q1 = 0;
				if ( le->qe >= 0 )
					q2 = x[le->qe];
				else
					q2 = 0;
				for ( j = 0; j < 7; j++ ) {
					ww += gi[j] * ( fa1[j] * q1 + fa2[j] * q2 );
				}
				vv += ww * ed->l / 2 * med[ed->medium].diel;
			}
	r[ed->portno - 1] += vv;   //Rij=Vji/Ii
	return;
}

Edge *searchborder ( Edge *ed )
{
	int i = 0;
	Edge *found = NULL;

	while ( i < borderedgenum ) {
		found = borderedge[i / 10];
		found = found + i % 10;
		i++;
		if ( !ptEqual ( ed->s->x, found->e->x ) )
			continue;
		if ( !ptEqual ( ed->s->y, found->e->y ) )
			continue;
		if ( !ptEqual ( ed->e->x, found->s->x ) )
			continue;
		if ( !ptEqual ( ed->e->y, found->s->y ) )
			continue;
		if ( found->edgetype == ed->edgetype && found->medium == ed->medium
				&& found->reversemed == ed->reversemed )
			break;
	}

	if ( found == NULL ) {
		printf ( "Can't found reverse edge of :\n" );
		printanedge ( ed );
	}
	else
		return ( found );
	assert ( 0 );
	return NULL;
}
int getUvar ( Medium *m, Vert *v, int no, Element *ele, EdgeType type )
{
	int i;
	Flag flag;
	double xe, ye;

	if ( v->varnum == -1 ) {
		v->varnum = v->edgenum;
		for ( i = 0; i < v->edgenum; i++ )
			if ( v->edge[i]->electype == FORCE )
				v->Uvarno = -v->edge[i]->portno;
		if ( v->Uvarno >= 0 ) {
			flag = 0;
			for ( i = 0; i < v->edgenum; i++ )
				if ( v->edge[i]->electype == BORDER
						&& v->edge[i]->medium == m->mediumno )
					flag = 1;
			if ( v->edge[no]->electype != BORDER && flag == 0 )
				v->Uvarno = gbvarno++;
			else {
				v->feedback = ( Element * ) malloc ( sizeof ( Element ) );
					v->feedback->type = LINELE;
					v->feedback->ele.le = ele->ele.le;
			}
			v->varnum++;
		}
		v->varnum = v->varnum - v->edgenum + v->forcenum + v->bordernum;
		calextra ( v );
	}
	//This vert had been handled before
	else if ( v->feedback != NULL ) {
		v->Uvarno = gbvarno++;
			xe = v->feedback->ele.le->xe;
			ye = v->feedback->ele.le->ye;
			if ( ptEqual ( xe, v->x ) && ptEqual ( ye, v->y ) )
				v->feedback->ele.le->ue = v->Uvarno;
			else
				v->feedback->ele.le->us = v->Uvarno;
		v->feedback = NULL;
	}
	return v->Uvarno;
}

int getQvar ( Vert *v, int no )
{
	//NATURAL
	if ( v->Qvarno[no] < -v->edgenum )
		return -1;
	//already
	if ( v->Qvarno[no] > 0 )
		return v->Qvarno[no];
	//need a q var
	if ( v->Qvarno[no] == -v->edgenum ) {
		v->Qvarno[no] = gbvarno++;
		return v->Qvarno[no];
	}
	else {
		v->Qvarno[-v->Qvarno[no]] = gbvarno;
		v->Qvarno[no] = gbvarno++;
	}
	return ( v->Qvarno[no] );
}

void calextra ( Vert *v )
{
	int i, j, medcnt;
	int m[10];

	medcnt = 0;
	for ( i = 0; i < v->edgenum; i++ ) {
		for ( j = 0; j < medcnt; j++ )
			if ( v->edge[i]->medium == m[j] )
				break;
		if ( j >= medcnt )
			m[medcnt++] = v->edge[i]->medium;
		if ( v->edge[i]->electype == BORDER ) {
			for ( j = 0; j < medcnt; j++ )
				if ( v->edge[i]->reversemed == m[j] )
					break;
			if ( j >= medcnt )
				m[medcnt++] = v->edge[i]->reversemed;
		}
	}
	//Form relative medium array about point v, as ascendent sequence
	//medcnt is number of relative mediums

	i = 0;
	j = 0;
	//Must add addition sp
	while ( v->varnum > medcnt ) {
		if ( v->edge[i]->electype == BORDER && v->varnum - medcnt == 1 )
			j = 1;
		if ( v->edge[i]->s == v )
			if ( j == 1 )
				v->edge[i]->stype = 2;
			else
				v->edge[i]->stype = 1;
		else if ( j == 1 )
			v->edge[i]->etype = 2;
		else
			v->edge[i]->etype = 1;
		if ( v->edge[i]->electype == BORDER && j != 1 ) {
			medcnt += 2;
			printpoint ( v );
		}
		else
			medcnt++;
		j = 0;
		i++;
	}
}

void countvarsimple ( Medium *m, Edge *ed )
{
	Element *ele = ( Element * ) malloc ( sizeof ( Element ) );
	LinEle *le;
	int i, j, sno, eno;
	EdgeType type;

	sno = ed->sno;
	eno = ed->eno;
	type = ed->edgetype;

	if ( ed->electype != BORDER || ed->medium == m->mediumno )
				ele->type = LINELE;
				switch ( ed->electype ) {
					case FORCE:
						for ( i = 0; i < ed->elenum; i++ ) {
							le = ed->ele + i;
							ele->ele.le = le;
							if ( i == 0 ) {
								le->us = getUvar ( m, ed->s, sno, ele, type );
								le->qs = getQvar ( ed->s, sno );
							}
							else {
								le->us = ed->ele[i - 1].ue;
								le->qs = ed->ele[i - 1].qe;
							}
							if ( i == ed->elenum - 1 ) {
								le->ue = getUvar ( m, ed->e, eno, ele, type );
								le->qe = getQvar ( ed->e, eno );
							}
							else {
								le->ue = -ed->portno;
								le->qe = gbvarno++;
							}
						}
						break;
					case NATURAL:
						for ( i = 0; i < ed->elenum; i++ ) {
							le = ed->ele + i;
							ele->ele.le = le;
							if ( i == 0 ) {
								le->us = getUvar ( m, ed->s, sno, ele, type );
								le->qs = getQvar ( ed->s, sno );
							}
							else {
								le->us = ed->ele[i - 1].ue;
								le->qs = -1;
							}
							if ( i == ed->elenum - 1 ) {
								le->ue = getUvar ( m, ed->e, eno, ele, type );
								le->qe = getQvar ( ed->e, eno );
							}
							else {
								le->ue = gbvarno++;
								le->qe = -1;
							}
						}
						break;
					default:
						break;
				}
	free ( ( char * ) ele );
}

void countvarborderu ( Medium *m, Edge *ed )
{
	Element *ele = ( Element * ) malloc ( sizeof ( Element ) );
	LinEle *le;
	int i, j, sno, eno;
	EdgeType type;

	sno = ed->sno;
	eno = ed->eno;
	type = ed->edgetype;

			ele->type = LINELE;
			for ( i = 0; i < ed->elenum; i++ ) {
				le = ed->ele + i;
				ele->ele.le = le;
				if ( i == 0 )
					le->us = getUvar ( m, ed->s, sno, ele, type );
				else
					le->us = ed->ele[i - 1].ue;
				if ( i == ed->elenum - 1 )
					le->ue = getUvar ( m, ed->e, eno, ele, type );
				else
					le->ue = gbvarno++;
			}
	free ( ( char * ) ele );
}

void countvarborderq ( Medium *m, Edge *ed )
{
	Element *ele = ( Element * ) malloc ( sizeof ( Element ) );
	LinEle *le;
	int i, j, sno, eno;
	EdgeType type;

	sno = ed->sno;
	eno = ed->eno;
	type = ed->edgetype;

			ele->type = LINELE;
			for ( i = 0; i < ed->elenum; i++ ) {
				le = ed->ele + i;
				ele->ele.le = le;
				if ( i == 0 )
					le->qs = getQvar ( ed->s, sno );
				else
					le->qs = ed->ele[i - 1].qe;
				if ( i == ed->elenum - 1 )
					le->qe = getQvar ( ed->e, eno );
				else
					le->qe = gbvarno++;
			}
	free ( ( char * ) ele );
}

void countedgenuminmed ( Medium *m, Edge *ed )
{
	ed->s->edgenum++;
	ed->e->edgenum++;
	if ( ed->s->edgenum == 4 )
		ed->stype = -1;
	if ( ed->e->edgenum == 4 )
		ed->etype = -1;
}

void createelement ( Medium *m, Edge *ed )
{
	LinEle *le;
	double xs, ys, as;
	int k;

	if ( ed->electype != BORDER || ed->medium == m->mediumno ) {
		ed->sno = ed->s->edgenum;
		ed->eno = ed->e->edgenum;
		ed->s->edge[ed->s->edgenum] = ed;
		ed->e->edge[ed->e->edgenum] = ed;
		if ( ed->electype == BORDER )
			ed->s->bordernum++;
		else if ( ed->electype == FORCE )
			ed->s->forcenum++;
		ed->s->edgenum++;
		if ( !samepoint ( ed->s, ed->e ) ) {
			ed->e->edgenum++;
			if ( ed->electype == BORDER )
				ed->e->bordernum++;
			else if ( ed->electype == FORCE )
				ed->e->forcenum++;
		}
		ed->s->edge =
			( Edge ** ) realloc ( ( char * ) ed->s->edge,
								  ( ed->s->edgenum + 1 ) * sizeof ( Edge * ) );
		ed->e->edge =
			( Edge ** ) realloc ( ( char * ) ed->e->edge,
								  ( ed->e->edgenum + 1 ) * sizeof ( Edge * ) );

		if ( ed->edgetype == LINE ) {
			ed->dx = ( ed->e->x - ed->s->x ) / ed->elenum;
			ed->dy = ( ed->e->y - ed->s->y ) / ed->elenum;
			ed->l = sqrt ( ed->dx * ed->dx + ed->dy * ed->dy );
			ed->ele = ( LinEle * ) malloc ( ed->elenum * sizeof ( LinEle ) );
			xs = ed->s->x;
			ys = ed->s->y;
			for ( k = 0; k < ed->elenum; k++ ) {
				le = ed->ele + k;
				le->xs = xs;
				le->ys = ys;
				xs += ed->dx;
				ys += ed->dy;
				le->xe = xs;
				le->ye = ys;
				le->gp1 = NULL;
				le->gp2 = NULL;
				le->gp3 = NULL;
				le->gp4 = NULL;
				le->gp5 = NULL;
				le->gp6 = NULL;
				le->gp7 = NULL;
				le->gp8 = NULL;
				le->gp12 = NULL;
				le->gp16 = NULL;
				le->gp20 = NULL;
				le->gp24 = NULL;
				le->gp32 = NULL;
			}
		}
	}
}

Bool samepoint ( Vert *s, Vert *e )
{
	if ( fabs ( s->x - e->x ) > POINTGAP )
		return FALSE;
	else if ( fabs ( s->y - e->y ) > POINTGAP )
		return FALSE;
	else
		return TRUE;
}

int newsp ( SourcePoint *p2, int no, double x, double y, int u1, int u2 )
{
	p2->no = spno++;
	p2->x = x;
	p2->y = y;
	p2->u1 = u1;
	p2->u2 = u2;
	return ( spno );
}

/*
If elenum=1
we can add additional start and end on a same arc at the same time,
but can't do that on an edge(they are the same source point)
*/
void buildsp ( Medium *m, SourcePoint *sp )
{
	int i, j;
	Edge *ed;
	SourcePoint *p2;
	LinEle *le;
	for ( i = 0; i < m->edgenum; i++ ) {
		ed = m->edge[i];
				le = ed->ele;
				if ( ed->electype == BORDER && ed->reversemed == m->mediumno ) {
					//additional start
					if ( ( ed->stype == 1 || ed->stype == 2 )
							&& ed->medium == m->mediumno ) {
						spno =
							newsp ( sp + spno, spno, ( 3 * le->xs + le->xe ) / 4,
									( 3 * le->ys + le->ye ) / 4, le->us, le->ue );
					}
					//start of element
					for ( j = 1; j < ed->elenum; j++ ) {
						le = ed->ele + j;
						spno =
							newsp ( sp + spno, spno, le->xs, le->ys, le->us,
									-condobjnum - 1 );
					}
					//additional end
					if ( ( ( ed->etype == 1 || ed->etype == 2 )
							&& ed->medium == m->mediumno ) ) {
						spno =
							newsp ( sp + spno, spno, ( le->xs + 3 * le->xe ) / 4,
									( le->ys + 3 * le->ye ) / 4, le->us, le->ue );
					}
					if ( ed->etype != -1 )
						spno =
							newsp ( sp + spno, spno, le->xe, le->ye, le->ue,
									-condobjnum - 1 );
					//end
				}
				//reverse side of BORDER
				else {
					//start
					if ( ed->stype != -1 )
						spno =
							newsp ( sp + spno, spno, le->xs, le->ys, le->us,
									-condobjnum - 1 );
					//additional start
					if ( ( ed->stype == 1 || ed->stype == 2 )
							&& ed->medium == m->mediumno ) {
						spno =
							newsp ( sp + spno, spno, ( 3 * le->xs + le->xe ) / 4,
									( 3 * le->ys + le->ye ) / 4, le->us, le->ue );
					}
					//start of element
					for ( j = 1; j < ed->elenum; j++ ) {
						le = ed->ele + j;
						spno =
							newsp ( sp + spno, spno, le->xs, le->ys, le->us,
									-condobjnum - 1 );
					}
					//additional end
					if ( ( ed->etype == 1 || ed->etype == 2 )
							&& ed->medium == m->mediumno ) {
						spno =
							newsp ( sp + spno, spno, ( le->xs + 3 * le->xe ) / 4,
									( le->ys + 3 * le->ye ) / 4, le->us, le->ue );
					}
				}
	}
}

double *calgpoint ( Edge *ed, LinEle *le, SourcePoint *sp, int *gpnum, int *k )
{
	double dxx, dyy, rr;
	double dx, dy, r;
	int i, n;
	double **gp = NULL;
	double *fa1;

	dxx = le->xs - sp->x;
	dyy = le->ys - sp->y;
	dx = le->xe - sp->x;
	dy = le->ye - sp->y;
	r = dx * dx + dy * dy;
	rr = dxx * dxx + dyy * dyy;
	if ( r < rr )
		rr = r;
	rr = ed->l * ed->l / rr;
	if ( rr < 16e-5 ) {
		n = 1;
		*k = 0;
		gp = & ( le->gp1 );
	}
	else if ( rr < 0.005 ) {
		n = 2;
		*k = 1;
		gp = & ( le->gp2 );
	}
	else if ( rr < 0.14 ) {
		n = 3;
		*k = 1 + 2;
		gp = & ( le->gp3 );
	}
	else if ( rr < 0.284 ) {
		n = 4;
		*k = 1 + 2 + 3;
		gp = & ( le->gp4 );
	}
	else if ( rr < 0.63 ) {
		n = 5;
		*k = 10;
		gp = & ( le->gp5 );
	}
	else if ( rr < 1.09 ) {
		n = 6;
		*k = 15;
		gp = & ( le->gp6 );
	}
	else if ( rr < 1.6 ) {
		n = 7;
		*k = 21;
		gp = & ( le->gp7 );
	}
	else if ( rr < 2.13 ) {
		n = 8;
		*k = 28;
		gp = & ( le->gp8 );
	}
	else if ( rr < 4.17 ) {
		n = 12;
		*k = 36;
		gp = & ( le->gp12 );
	}
	else if ( rr < 5.84 ) {
		n = 16;
		*k = 48;
		gp = & ( le->gp16 );
	}
	else if ( rr < 7.14 ) {
		n = 20;
		*k = 64;
		gp = & ( le->gp20 );
	}
	else if ( rr < 8.17 ) {
		n = 24;
		*k = 84;
		gp = & ( le->gp24 );
	}
	else {
		n = 32;
		*k = 108;
		gp = & ( le->gp32 );
	}
	if ( *gp == NULL ) {
		*gp = ( double * ) malloc ( 2 * n * sizeof ( double ) );
		fa1 = fa11 + *k;
		for ( i = 0; i < n; i++ ) {
			( *gp ) [i * 2] = le->xs + fa1[i] * ed->dx;
			( *gp ) [2 * i + 1] = le->ys + fa1[i] * ed->dy;
		}
	}
	*gpnum = n;
	return ( *gp );
}

void setgvalue()
{
	int i;

	//1
	g00[0] = 0;
	gic[0] = 2;

	//2
	g00[1 + 0] = -0.577350269189626;
	g00[1 + 1] = -g00[1 + 0];
	gic[1] = gic[2] = 1.0;

	//3
	g00[3 + 0] = -0.774596669241483;
	g00[3 + 1] = -g00[3 + 0];
	g00[3 + 2] = 0.0;
	gic[3 + 0] = gic[3 + 1] = 0.555555555555556;
	gic[3 + 2] = 0.888888888888889;

	//4
	g00[6 + 1] = 0.861136311594053;
	g00[6 + 2] = -0.861136311594053;
	g00[6 + 3] = 0.339981043584856;
	g00[6 + 0] = -g00[6 + 3];
	gic[6 + 1] = gic[6 + 2] = 0.347854845137454;
	gic[6 + 0] = gic[6 + 3] = 0.652145154862546;

	//5
	g00[10 + 0] = 0.0;
	g00[10 + 1] = 0.906179845938664;
	g00[10 + 2] = -g00[10 + 1];
	g00[10 + 3] = 0.538469310105683;
	g00[10 + 4] = -g00[10 + 3];
	gic[10] = 0.568888888888889;
	gic[10 + 1] = gic[10 + 2] = 0.236926885056189;
	gic[10 + 3] = gic[10 + 4] = 0.478628670499366;

	//6
	g00[15] = -0.238619186083197;
	g00[15 + 1] = 0.932469514203152;
	g00[15 + 2] = -g00[15 + 1];
	g00[15 + 3] = 0.661209386466265;
	g00[15 + 4] = -g00[15 + 3];
	g00[15 + 5] = -g00[15];
	gic[15] = gic[15 + 5] = 0.467913934572691;
	gic[15 + 1] = gic[15 + 2] = 0.171324492379170;
	gic[15 + 3] = gic[15 + 4] = 0.360761573048139;

	//7
	g00[21 + 0] = 0.0;
	g00[21 + 1] = 0.405845151377397;
	g00[21 + 2] = -0.405845151377397;
	g00[21 + 3] = 0.741531185599394;
	g00[21 + 4] = -0.741531185599394;
	g00[21 + 5] = 0.949107912342759;
	g00[21 + 6] = -0.949107912342759;
	gic[21 + 0] = 0.417959183673469;
	gic[21 + 1] = 0.381830050505119;
	gic[21 + 2] = 0.381830050505119;
	gic[21 + 3] = 0.279705391489277;
	gic[21 + 4] = 0.279705391489277;
	gic[21 + 5] = 0.129484966168870;
	gic[21 + 6] = 0.129484966168870;

	//8
	g00[28 + 0] = -0.183434642495650;
	g00[28 + 1] = 0.960289856497536;
	g00[28 + 2] = -g00[28 + 1];
	g00[28 + 3] = 0.796666477413627;
	g00[28 + 4] = -g00[28 + 3];
	g00[28 + 5] = 0.525532409916329;
	g00[28 + 6] = -g00[28 + 5];
	g00[28 + 7] = -g00[28 + 0];
	gic[28 + 0] = gic[28 + 7] = 0.362683783378362;
	gic[28 + 1] = gic[28 + 2] = 0.101228536290376;
	gic[28 + 3] = gic[28 + 4] = 0.222381034453374;
	gic[28 + 5] = gic[28 + 6] = 0.313706645877887;

	//12
	g00[36 + 0] = 0.981560634246719;
	g00[36 + 1] = -g00[36 + 0];
	g00[36 + 2] = 0.904117256370475;
	g00[36 + 3] = -g00[36 + 2];
	g00[36 + 4] = 0.769902674194305;
	g00[36 + 5] = -g00[36 + 4];
	g00[36 + 6] = 0.587317954286617;
	g00[36 + 7] = -g00[36 + 6];
	g00[36 + 8] = 0.367831498998180;
	g00[36 + 9] = -g00[36 + 8];
	g00[36 + 10] = 0.125233408511469;
	g00[36 + 11] = -g00[36 + 10];
	gic[36 + 0] = gic[36 + 1] = 0.047175336386512;
	gic[36 + 2] = gic[36 + 3] = 0.106939325995318;
	gic[36 + 4] = gic[36 + 5] = 0.160078328543346;
	gic[36 + 6] = gic[36 + 7] = 0.203167426723066;
	gic[36 + 8] = gic[36 + 9] = 0.233492536538355;
	gic[36 + 10] = gic[36 + 11] = 0.249147045813403;

	//16
	g00[48 + 0] = 0.989400934991649932596;
	g00[48 + 1] = -g00[48 + 0];
	g00[48 + 2] = 0.944575023073232576078;
	g00[48 + 3] = -g00[48 + 2];
	g00[48 + 4] = 0.865631202387831743880;
	g00[48 + 5] = -g00[48 + 4];
	g00[48 + 6] = 0.755404408355003033895;
	g00[48 + 7] = -g00[48 + 6];
	g00[48 + 8] = 0.617876244402643748447;
	g00[48 + 9] = -g00[48 + 8];
	g00[48 + 10] = 0.458016777657227386342;
	g00[48 + 11] = -g00[48 + 10];
	g00[48 + 12] = 0.281603550779258913230;
	g00[48 + 13] = -g00[48 + 12];
	g00[48 + 14] = 0.095012509837637440185;
	g00[48 + 15] = -g00[48 + 14];
	gic[48 + 0] = gic[48 + 1] = 0.027152459411754094852;
	gic[48 + 2] = gic[48 + 3] = 0.062253523938647892863;
	gic[48 + 4] = gic[48 + 5] = 0.095158511682492784810;
	gic[48 + 6] = gic[48 + 7] = 0.124628971255533872052;
	gic[48 + 8] = gic[48 + 9] = 0.149595988816576732081;
	gic[48 + 10] = gic[48 + 11] = 0.169156519395002538189;
	gic[48 + 12] = gic[48 + 13] = 0.182603415044923588867;
	gic[48 + 14] = gic[48 + 15] = 0.189450610455068496285;

	//20
	g00[64 + 0] = 0.993128599185094924786;
	g00[64 + 1] = -g00[64 + 0];
	g00[64 + 2] = 0.963971927277913791268;
	g00[64 + 3] = -g00[64 + 2];
	g00[64 + 4] = 0.912234428251325905868;
	g00[64 + 5] = -g00[64 + 4];
	g00[64 + 6] = 0.839116971822218823395;
	g00[64 + 7] = -g00[64 + 6];
	g00[64 + 8] = 0.746331906460150792614;
	g00[64 + 9] = -g00[64 + 8];
	g00[64 + 10] = 0.636053680726515025453;
	g00[64 + 11] = -g00[64 + 10];
	g00[64 + 12] = 0.510867001950827098004;
	g00[64 + 13] = -g00[64 + 12];
	g00[64 + 14] = 0.373706088715419560673;
	g00[64 + 15] = -g00[64 + 14];
	g00[64 + 16] = 0.227785851141645078080;
	g00[64 + 17] = -g00[64 + 16];
	g00[64 + 18] = 0.076526521133497333755;
	g00[64 + 19] = -g00[64 + 18];
	gic[64 + 0] = gic[64 + 1] = 0.017614007139152118312;
	gic[64 + 2] = gic[64 + 3] = 0.040601429800386941331;
	gic[64 + 4] = gic[64 + 5] = 0.062672048334109063570;
	gic[64 + 6] = gic[64 + 7] = 0.083276741576704748725;
	gic[64 + 8] = gic[64 + 9] = 0.101930119817240435037;
	gic[64 + 10] = gic[64 + 11] = 0.118194531961518417312;
	gic[64 + 12] = gic[64 + 13] = 0.131688638449176626898;
	gic[64 + 14] = gic[64 + 15] = 0.142096109318382051329;
	gic[64 + 16] = gic[64 + 17] = 0.149172986472603746788;
	gic[64 + 18] = gic[64 + 19] = 0.152753387130725850698;

	//24
	g00[84 + 0] = 0.995187219997021360180;
	g00[84 + 1] = -g00[84 + 0];
	g00[84 + 2] = 0.974728555971309498198;
	g00[84 + 3] = -g00[84 + 2];
	g00[84 + 4] = 0.938274552002732758524;
	g00[84 + 5] = -g00[84 + 4];
	g00[84 + 6] = 0.886415527004401034213;
	g00[84 + 7] = -g00[84 + 6];
	g00[84 + 8] = 0.820001985973902921954;
	g00[84 + 9] = -g00[84 + 8];
	g00[84 + 10] = 0.740124191578554364244;
	g00[84 + 11] = -g00[84 + 10];
	g00[84 + 12] = 0.648093651936975569252;
	g00[84 + 13] = -g00[84 + 12];
	g00[84 + 14] = 0.545421471388839535656;
	g00[84 + 15] = -g00[84 + 14];
	g00[84 + 16] = 0.433793507626045138487;
	g00[84 + 17] = -g00[84 + 16];
	g00[84 + 18] = 0.315042679696163374387;
	g00[84 + 19] = -g00[84 + 18];
	g00[84 + 20] = 0.191118867473616309159;
	g00[84 + 21] = -g00[84 + 20];
	g00[84 + 22] = 0.064056892862605626085;
	g00[84 + 23] = -g00[84 + 22];
	gic[84 + 0] = gic[84 + 1] = 0.012341229799987199547;
	gic[84 + 2] = gic[84 + 3] = 0.028531388628933663181;
	gic[84 + 4] = gic[84 + 5] = 0.044277438817419806169;
	gic[84 + 6] = gic[84 + 7] = 0.059298584915436780746;
	gic[84 + 8] = gic[84 + 9] = 0.073346481411080305734;
	gic[84 + 10] = gic[84 + 11] = 0.086190161531953275917;
	gic[84 + 12] = gic[84 + 13] = 0.097618652104113888270;
	gic[84 + 14] = gic[84 + 15] = 0.107444270115965634783;
	gic[84 + 16] = gic[84 + 17] = 0.115505668053725601353;
	gic[84 + 18] = gic[84 + 19] = 0.121670472927803391204;
	gic[84 + 20] = gic[84 + 21] = 0.125837456346828296121;
	gic[84 + 22] = gic[84 + 23] = 0.127938195346752156974;

	//32
	g00[108 + 0] = 0.997263861849481563545;
	g00[108 + 1] = -g00[108 + 0];
	g00[108 + 2] = 0.985611511545268335400;
	g00[108 + 3] = -g00[108 + 2];
	g00[108 + 4] = 0.964762255587506430774;
	g00[108 + 5] = -g00[108 + 4];
	g00[108 + 6] = 0.934906075937739689171;
	g00[108 + 7] = -g00[108 + 6];
	g00[108 + 8] = 0.896321155766052123965;
	g00[108 + 9] = -g00[108 + 8];
	g00[108 + 10] = 0.849367613732569970134;
	g00[108 + 11] = -g00[108 + 10];
	g00[108 + 12] = 0.794483795967942406963;
	g00[108 + 13] = -g00[108 + 12];
	g00[108 + 14] = 0.732182118740289680387;
	g00[108 + 15] = -g00[108 + 14];
	g00[108 + 16] = 0.663044266930215200975;
	g00[108 + 17] = -g00[108 + 16];
	g00[108 + 18] = 0.587715757240762329041;
	g00[108 + 19] = -g00[108 + 18];
	g00[108 + 20] = 0.506899908932229390024;
	g00[108 + 21] = -g00[108 + 20];
	g00[108 + 22] = 0.421351276130635345364;
	g00[108 + 23] = -g00[108 + 22];
	g00[108 + 24] = 0.331868602282127649780;
	g00[108 + 25] = -g00[108 + 24];
	g00[108 + 26] = 0.239287362252137074545;
	g00[108 + 27] = -g00[108 + 26];
	g00[108 + 28] = 0.144471961582796493485;
	g00[108 + 29] = -g00[108 + 28];
	g00[108 + 30] = 0.048307665687738316235;
	g00[108 + 31] = -g00[108 + 30];
	gic[108 + 0] = gic[108 + 1] = 0.007018610009470096600;
	gic[108 + 2] = gic[108 + 3] = 0.016274394730905670605;
	gic[108 + 4] = gic[108 + 5] = 0.025392065309262059456;
	gic[108 + 6] = gic[108 + 7] = 0.034273862913021433103;
	gic[108 + 8] = gic[108 + 9] = 0.042835898022226680657;
	gic[108 + 10] = gic[108 + 11] = 0.050998059262376176196;
	gic[108 + 12] = gic[108 + 13] = 0.058684093478535547145;
	gic[108 + 14] = gic[108 + 15] = 0.065822222776361846838;
	gic[108 + 16] = gic[108 + 17] = 0.072345794108848506225;
	gic[108 + 18] = gic[108 + 19] = 0.078193895787070306472;
	gic[108 + 20] = gic[108 + 21] = 0.083311924226946755222;
	gic[108 + 22] = gic[108 + 23] = 0.087652093004403811143;
	gic[108 + 24] = gic[108 + 25] = 0.091173878695763884713;
	gic[108 + 26] = gic[108 + 27] = 0.093844399080804565639;
	gic[108 + 28] = gic[108 + 29] = 0.095638720079274859419;
	gic[108 + 30] = gic[108 + 31] = 0.096540088514727800567;

	for ( i = 0; i < GAUSSTOTAL; i++ ) {
		//for 1 order E.
		fa11[i] = ( 1.0 - g00[i] ) / 2.0;
		fa12[i] = ( 1.0 + g00[i] ) / 2.0;
	}
}

Bool fullmatrix ( double **b, double **x, double **r )
{
	int i, j, k;
	Edge *ed;
	SourcePoint *sp;
	int portnum, varnum, linenum;
	double **a = NULL;
	double *abx = NULL;

	for ( i = 0; i < mednum; i++ ) {
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			if ( ed->electype == BORDER )
				if ( ed->reversemed == i && ed->medium < i )
					countvarborderu ( med + i, ed );
		}
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			if ( ed->electype != BORDER )
				countvarsimple ( med + i, ed );
		}
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
			if ( ed->electype == BORDER )
				if ( ed->medium == i && ed->reversemed > i )
					countvarborderq ( med + i, ed );
		}
	}

	for ( i = 0; i < mednum; i++ )
		for ( j = 0; j < med[i].edgenum; j++ ) {
			ed = med[i].edge[j];
		}

	varnum = gbvarno;
	portnum = condobjnum;
	linenum = 0;

	abx =
		( double * ) malloc ( ( 2 * gbvarno * condobjnum + gbvarno * gbvarno ) *
							  sizeof ( double ) );
	memset ( abx, 0,
			 ( 2 * gbvarno * condobjnum + gbvarno * gbvarno ) * sizeof ( double ) );
	a = ( double ** ) malloc ( gbvarno * sizeof ( double * ) );
	if ( abx == NULL || a == NULL || b == NULL || x == NULL ) {
		printf ( "Out of memory\n" );
		exit ( 3 );
	}
	for ( i = 0; i < gbvarno; i++ ) {
		a[i] = abx;
		abx += gbvarno;
	}
	for ( i = 0; i < condobjnum; i++ ) {
		b[i] = abx;
		abx += gbvarno;
	}
	for ( i = 0; i < condobjnum; i++ ) {
		x[i] = abx;
		abx += gbvarno;
	}
	abx = a[0];

	sp = ( SourcePoint * ) malloc ( varnum * sizeof ( SourcePoint ) );
	for ( i = 0; i < mednum; i++ ) {
		linenum = integral ( med + i, a[linenum], b, sp, linenum );
	}
	int zeronum = 0;
	for ( i = 0; i < gbvarno; i++ )
		if ( Equal ( a[i][i], 0 ) && Equal ( a[i + 1][i], 0 ) )
			zeronum++;

	solveequation ( a, b, x );
	result ( x, r );

	free ( ( char * ) a );
	free ( ( char * ) abx );
	free ( ( char * ) sp );
	return ( varnum == spno );
}

void result ( double **x, double **r )
{
	int i, j, k;

	//zcs.  condobjnum=ports num;mednum=1;edgenum=line num;nametable=ports name;
	for ( i = 0; i < condobjnum; i++ ){
		if ( calcarray[i] == 1 )
			for ( j = 0; j < mednum; j++ ){
				for ( k = 0; k < med[j].edgenum; k++ ){
					I_integral ( x[i], r[i], med[j].edge[k] );
				}
			}
	}
	out ( r );
}
