#include <stdlib.h>
#include <assert.h>
#include "type.h"
#include "shape.h"
#include "bem.h"
#include "globalvar.h"

void initobj ( Object *obj )
{
	obj->objno = -1;
	obj->electype = NONE;
	obj->shellnum = 0;
	obj->shell = NULL;
	obj->voltval = INVALIDVOLT;
	obj->dielval = INVALIDDIEL;
	obj->rhoval = INVALIDRHO;
}

void copyshell ( Shell *o, Shell *n )
{
	n->maxx = o->maxx;
	n->maxy = o->maxy;
	n->minx = o->minx;
	n->miny = o->miny;
	n->shapetype = o->shapetype;
}

void copypoly ( Poly *o, Poly *n )
{
	int i;

	n->vertnum = o->vertnum;
	n->vertex = ( Point * ) malloc ( n->vertnum * sizeof ( Point ) );
	n->eles = ( int * ) malloc ( n->vertnum * sizeof ( int ) );
	for ( i = 0; i < n->vertnum; i++ ) {
		n->vertex[i] = o->vertex[i];
		n->eles[i] = o->eles[i];
	}
}

void copypoints ( Points *o, Points *n )
{
	int i;

	n->vertnum = o->vertnum;
	n->d1 = o->d1;
	n->d2 = o->d2;
	n->vertex = ( Point * ) malloc ( n->vertnum * sizeof ( Point ) );
	n->eles = ( int * ) malloc ( n->vertnum * sizeof ( int ) );
	for ( i = 0; i < n->vertnum; i++ ) {
		n->vertex[i] = o->vertex[i];
		n->eles[i] = o->eles[i];
	}
}

void printobj ( Object *obj )
{
	int i;

	//printf("no=%d,name=%s,have %3d shapes\n",obj->objno,obj->name,obj->shellnum);
	if ( obj->electype == MEDIUM )
		if ( obj->dielval != INVALIDDIEL )
			printf ( "This obj is a medium,diel=%f\n", obj->dielval );
		else
			printf ( "Error! This obj has no valid diel value\n" );
	else if ( obj->electype == CONDUCTOR )
		if ( obj->voltval != INVALIDVOLT )
			printf ( "This obj is a conductor,volt=%f\n", obj->voltval );
		else
			printf ( "Error ! This obj has no valid volt value\n" );
	else
		printf ( "Error!! This object has no electype!\n" );
	for ( i = 0; i < obj->shellnum; i++ )
		printshell ( obj->shell[i] );
}

void printshell ( Shell *shell )
{
	//printf("Shell %d locate from (%6.2f,%6.2f) to (%6.2f,%6.2f)\n",shell->shellno,shell->minx,shell-miny,shell->maxx,shell->maxy);
	switch ( shell->shapetype ) {
		case BOX:
			printbox ( shell->shape.bbox );
			break;
		case POLY:
			printpoly ( shell->shape.ppoly );
			break;
		case EDGES:
			printf ( "There's no more information of edges\n" );
			break;
		default:
			break;
	}
}

void printbox ( Box *box )
{
	printf ( "Now is the information of a box\n" );
	printf
	( "Four vertex are: (%6.2f,%6.2f),(%6.2f,%6.2f),(%6.2f,%6.2f),(%6.2f,%6.2f)\n",
	  box->x1, box->y1, box->x2, box->y2, box->x3, box->y3, box->x4,
	  box->y4 );
	printf ( "Width=%6.2f,height=%6.2f\n", box->width, box->height );
	printf ( "Divide the width to %3d elements, height to %3d\n\n",
			 box->elesonwidth, box->elesonheight );
}

void printpoly ( Poly *poly )
{
	int i;
	printf ( "Now is the information of a poly: several connective lines\n" );
	for ( i = 0; i < poly->vertnum; i++ )
		printf ( "(%6.2f,%6.2f) ", poly->vertex[i].x, poly->vertex[i].y );
	printf ( "\n" );
	printf ( "Divide the lines to" );
	for ( i = 0; i < poly->vertnum - 1; i++ )
		printf ( "%d, ", poly->eles[i] );
	printf ( "elements\n\n" );
}

void setboxshell ( Shell *shell, Box *box )
{
	shell->maxx = box->x2;
	shell->maxy = box->y3;
	shell->minx = box->x4;
	shell->miny = box->y1;

	if ( box->x1 > shell->maxx )
		shell->maxx = box->x1;
	else if ( box->x1 < shell->minx )
		shell->minx = box->x1;
	if ( box->x2 < shell->minx )
		shell->minx = box->x2;
	if ( box->x3 > shell->maxx )
		shell->maxx = box->x3;
	else if ( box->x3 < shell->minx )
		shell->minx = box->x3;
	if ( box->x4 > shell->maxx )
		shell->maxx = box->x4;

	if ( box->y1 > shell->maxy )
		shell->maxy = box->y1;
	if ( box->y2 > shell->maxy )
		shell->maxy = box->y2;
	else if ( box->y2 < shell->miny )
		shell->miny = box->y2;
	if ( box->y3 < shell->miny )
		shell->miny = box->y3;
	if ( box->y4 > shell->maxy )
		shell->maxy = box->y4;
	else if ( box->y4 < shell->miny )
		shell->miny = box->y4;
}

void setpolyshell ( Shell *shell, Poly *poly )
{
	int i;

	shell->maxx = shell->minx = poly->vertex[0].x;
	shell->maxy = shell->miny = poly->vertex[0].y;
	for ( i = 1; i < poly->vertnum; i++ ) {
		if ( shell->maxx < poly->vertex[i].x )
			shell->maxx = poly->vertex[i].x;
		else if ( shell->minx > poly->vertex[i].x )
			shell->minx = poly->vertex[i].x;
		if ( shell->maxy < poly->vertex[i].y )
			shell->maxy = poly->vertex[i].y;
		else if ( shell->miny > poly->vertex[i].y )
			shell->miny = poly->vertex[i].y;
	}
}

FloatValueRelation fltvalrela ( double x, double y )
{
	if ( Greater ( x, y ) )
		return ( GREATER );
	else if ( Equal ( x, y ) )
		return ( EQUAL );
	else
		return ( LESS );
}

void printtwoshapesrelation ( ShellRelation sr )
{
	switch ( sr ) {
		case SEPERATE:
			printf ( "  SEPE" );
			break;
		case IN:
			printf ( "    IN" );
			break;
		case OUT:
			printf ( "   OUT" );
			break;
		case ADJOIN:
			printf ( "   ADJ" );
			break;
		case MERGE:
			printf ( " MERGE" );
			break;
		case MERGEIN:
			printf ( "   MIN" );
			break;
		case MERGEOUT:
			printf ( "  MOUT" );
			break;
		case INTERSECT:
			printf ( " INTER" );
			break;
	}
}

void printrelation ( ShellRelation **s, int n )
{
	int i, j;

	for ( i = 0; i < n; i++ ) {
		for ( j = 0; j < i; j++ )
			printf ( "      " );
		for ( j = i + 1; j < n; j++ )
			printtwoshapesrelation ( s[i][j - i - 1] );
		printf ( "\n" );
	}
}

ShellRelation setshellrelation ( Shell *s1, Shell *s2 )
{
	if ( ptGreater ( s1->minx, s2->maxx ) )
		return ( SEPERATE );
	if ( ptGreater ( s1->miny, s2->maxy ) )
		return ( SEPERATE );
	if ( ptGreater ( s2->minx, s1->maxx ) )
		return ( SEPERATE );
	if ( ptGreater ( s2->miny, s1->maxy ) )
		return ( SEPERATE );

	switch ( fltvalrela ( s1->maxx, s2->maxx ) ) {
		case GREATER:
			switch ( fltvalrela ( s1->minx, s2->maxx ) ) {
				case GREATER:
					return ( SEPERATE );
				case EQUAL:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else
						return ( ADJOIN );
					break;
				case LESS:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else if ( ptEqual ( s1->miny, s2->maxy )
							  || ptEqual ( s1->maxy, s2->miny ) )
						return ( ADJOIN );
					else
						switch ( fltvalrela ( s1->minx, s2->minx ) ) {
							case GREATER:
								if ( ptEqual ( s1->miny, s2->miny )
										|| ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGE );
								else
									return ( INTERSECT );
							case EQUAL:
								if ( ptGreater ( s1->maxy, s2->maxy )
										&& ptLess ( s1->miny, s2->miny ) )
									return ( MERGEOUT );
								else
									return ( MERGE );
							case LESS:
								if ( ptEqual ( s1->maxy, s2->maxy )
										|| ptEqual ( s1->miny, s2->miny ) )
									return ( MERGEOUT );
								else if ( ptGreater ( s1->maxy, s2->maxy )
										  && ptLess ( s1->miny, s2->miny ) )
									return ( OUT );
								else
									return ( INTERSECT );
								break;
						}
			}
		case EQUAL:
			if ( ptGreater ( s1->miny, s2->maxy ) || ptGreater ( s2->miny, s1->maxy ) )
				return ( SEPERATE );
			else if ( ptEqual ( s1->miny, s2->maxy ) || ptEqual ( s1->maxy, s2->miny ) )
				return ( ADJOIN );
			else if ( !ptLess ( s1->miny, s2->miny ) && !ptGreater ( s1->maxy, s2->maxy )
					  && !ptLess ( s1->minx, s2->minx ) )
				return ( MERGEIN );
			else if ( !ptGreater ( s1->miny, s2->miny ) && !ptLess ( s1->maxy, s2->maxy )
					  && !ptGreater ( s1->minx, s2->minx ) )
				return ( MERGEOUT );
			else
				return ( MERGE );
		case LESS:
			switch ( fltvalrela ( s1->maxx, s2->minx ) ) {
				case GREATER:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else if ( ptEqual ( s1->miny, s2->maxy )
							  || ptEqual ( s1->maxy, s2->miny ) )
						return ( ADJOIN );
					else
						switch ( fltvalrela ( s1->minx, s2->minx ) ) {
							case GREATER:
								if ( ptLess ( s1->miny, s2->miny )
										|| ptLess ( s2->maxy, s1->maxy ) )
									return ( INTERSECT );
								else if ( ptEqual ( s1->miny, s2->miny )
										  || ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGEIN );
								else
									return ( IN );
								break;
							case EQUAL:
								if ( ptLess ( s1->miny, s2->miny )
										|| ptLess ( s2->maxy, s1->maxy ) )
									return ( MERGE );
								else
									return ( MERGEIN );
								break;
							case LESS:
								if ( ptEqual ( s1->miny, s2->miny )
										|| ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGE );
								else
									return ( INTERSECT );
								break;
						}
				case EQUAL:
					if ( ptGreater ( s1->maxy, s2->miny ) || ptLess ( s1->miny, s2->maxy ) )
						return ( ADJOIN );
					else
						return ( SEPERATE );
				case LESS:
					return ( SEPERATE );
			}
	}
	assert ( 0 );
	return SEPERATE;
}

ShellRelation setobjectrelation ( Object *s1, Object *s2 )
{
	if ( ptGreater ( s1->minx, s2->maxx ) )
		return ( SEPERATE );
	if ( ptGreater ( s1->miny, s2->maxy ) )
		return ( SEPERATE );
	if ( ptGreater ( s2->minx, s1->maxx ) )
		return ( SEPERATE );
	if ( ptGreater ( s2->miny, s1->maxy ) )
		return ( SEPERATE );

	switch ( fltvalrela ( s1->maxx, s2->maxx ) ) {
		case GREATER:
			switch ( fltvalrela ( s1->minx, s2->maxx ) ) {
				case GREATER:
					return ( SEPERATE );
				case EQUAL:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else
						return ( ADJOIN );
					break;
				case LESS:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else if ( ptEqual ( s1->miny, s2->maxy )
							  || ptEqual ( s1->maxy, s2->miny ) )
						return ( ADJOIN );
					else
						switch ( fltvalrela ( s1->minx, s2->minx ) ) {
							case GREATER:
								if ( ptEqual ( s1->miny, s2->miny )
										|| ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGE );
								else
									return ( INTERSECT );
							case EQUAL:
								if ( ptGreater ( s1->maxy, s2->maxy )
										&& ptLess ( s1->miny, s2->miny ) )
									return ( MERGEOUT );
								else
									return ( MERGE );
							case LESS:
								if ( ptEqual ( s1->maxy, s2->maxy )
										|| ptEqual ( s1->miny, s2->miny ) )
									return ( MERGEOUT );
								else if ( ptGreater ( s1->maxy, s2->maxy )
										  && ptLess ( s1->miny, s2->miny ) )
									return ( OUT );
								else
									return ( INTERSECT );
								break;
						}
			}
		case EQUAL:
			if ( ptGreater ( s1->miny, s2->maxy ) || ptGreater ( s2->miny, s1->maxy ) )
				return ( SEPERATE );
			else if ( ptEqual ( s1->miny, s2->maxy ) || ptEqual ( s1->maxy, s2->miny ) )
				return ( ADJOIN );
			else if ( !ptLess ( s1->miny, s2->miny ) && !ptGreater ( s1->maxy, s2->maxy )
					  && !ptLess ( s1->minx, s2->minx ) )
				return ( MERGEIN );
			else if ( !ptGreater ( s1->miny, s2->miny ) && !ptLess ( s1->maxy, s2->maxy )
					  && !ptGreater ( s1->minx, s2->minx ) )
				return ( MERGEOUT );
			else
				return ( MERGE );
		case LESS:
			switch ( fltvalrela ( s1->maxx, s2->minx ) ) {
				case GREATER:
					if ( ptGreater ( s1->miny, s2->maxy )
							|| ptGreater ( s2->miny, s1->maxy ) )
						return ( SEPERATE );
					else if ( ptEqual ( s1->miny, s2->maxy )
							  || ptEqual ( s1->maxy, s2->miny ) )
						return ( ADJOIN );
					else
						switch ( fltvalrela ( s1->minx, s2->minx ) ) {
							case GREATER:
								if ( ptLess ( s1->miny, s2->miny )
										|| ptLess ( s2->maxy, s1->maxy ) )
									return ( INTERSECT );
								else if ( ptEqual ( s1->miny, s2->miny )
										  || ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGEIN );
								else
									return ( IN );
								break;
							case EQUAL:
								if ( ptLess ( s1->miny, s2->miny )
										|| ptLess ( s2->maxy, s1->maxy ) )
									return ( MERGE );
								else
									return ( MERGEIN );
								break;
							case LESS:
								if ( ptEqual ( s1->miny, s2->miny )
										|| ptEqual ( s1->maxy, s2->maxy ) )
									return ( MERGE );
								else
									return ( INTERSECT );
								break;
						}
				case EQUAL:
					if ( ptGreater ( s1->maxy, s2->miny ) || ptLess ( s1->miny, s2->maxy ) )
						return ( ADJOIN );
					else
						return ( SEPERATE );
				case LESS:
					return ( SEPERATE );
			}
	}
	return MERGEIN;
}

int *countshape ( char *filename )
{
	FILE *fp;
	char ch, str[15];
	int i, len;
	int no = 0, *num = NULL;
	int segment = 0;

	_edgeCount = 0;  //yuwj added
	int pntCount = 0;

	if ( ( fp = fopen ( filename, "r" ) ) == NULL ) {
		printf ( "cannot open file \n" );
		exit ( 0 );
	}

	ch = fgetc ( fp );
	while ( ch != EOF ) {
		while ( ( ch == ' ' ) || ( ch == '\n' ) || ( ch == '\r' ) || ( ch == '\t' ) ) {
			ch = fgetc ( fp );
		}
		if ( ch == EOF )
			break;
		i = 0;
		do {
			str[i] = ch;
			ch = fgetc ( fp );
			i++;
		}
		while ( ( ( ch <= 'z' ) && ( ch >= 'a' ) ) || ( ( ch <= '9' ) && ( ch >= '0' ) ) );
		if ( i < 15 )
			str[i] = '\0';
		if ( strcmp ( "end", str ) == 0 )
			segment++;
	}	
	
	//num of begin ... end
	num = ( int * ) malloc ( segment * sizeof ( int ) );
	memset ( num, 0, segment * sizeof ( int ) );

	rewind ( fp );
	ch = fgetc ( fp );
	while ( ch != EOF ) {
		while ( ( ch == ' ' ) || ( ch == '\n' ) || ( ch == '\r' ) || ( ch == '\t' ) ) {
			ch = fgetc ( fp );
		}
		if ( ch == EOF )
			break;

		if ( ch == '(' )    //yuwj added
			pntCount++;

		i = 0;
		do {
			str[i] = ch;
			ch = fgetc ( fp );
			i++;
		}
		while ( ( ( ch <= 'z' ) && ( ch >= 'a' ) ) || ( ( ch <= '9' ) && ( ch >= '0' ) ) );
		if ( i < 15 )
			str[i] = '\0';

		len = strlen ( str );
		switch ( len ) {
			case 3:
				if ( strcmp ( "box", str ) == 0 ) {
					num[no]++;
					_edgeCount = _edgeCount + 4;    //yuwj added
				}
				if ( strcmp ( "end", str ) == 0 )
					no++;
				break;
			case 4:
				if ( strcmp ( "poly", str ) == 0 )
					num[no]++;
				else if ( strcmp ( "copy", str ) == 0 )
					num[no] += MAX_SHAPE_IN_COPY;
				break;

		}
	}

	//printf("pntCount is %d\n",pntCount);
	_edgeCount = _edgeCount + pntCount;    //yuwj added
	for ( i = 0; i < segment; i++ )
		_edgeCount = _edgeCount - num[i];
	//printf("Total edge number is _edgeCount: %d\n", _edgeCount);

	fclose ( fp );
	return ( num );
}
